import React, {
    Component,
} from 'react';
import PropTypes from 'prop-types';

import {
    StyleSheet,
    View,
    Text,
    Animated,
    Dimensions,
    TouchableWithoutFeedback,
    Easing,
    Keyboard,
    Image,
} from 'react-native';
import RootSiblings from './SiblingsManager';
import LoadingView from './LoadingView';

class Progress extends Component {
    static displayName = 'Progress';
    // static positions = positions;
    // static durations = durations;

    static show = (message, options = {}) => {
        return new RootSiblings(<ProgressContainer
            {...options}
            visible={true}
        >
            {message}
        </ProgressContainer>);
    };

    static hide = progress => {
        if (progress instanceof RootSiblings) {
            progress.destroy();
        } else {
            console.warn(`progress.hide expected a \`RootSiblings\` instance as argument.\nBut got \`${typeof progress}\` instead.`);
        }
    };

    _progress = null;

    componentWillMount = () => {
        this._progress = new RootSiblings(<ProgressContainer
            {...nextProps}
            duration={0}
        />);
        console.log("ProgressContainer componentWillMount");
    };

    componentWillReceiveProps = nextProps => {
        this._progress.update(<ProgressContainer
            {...nextProps}
            duration={0}
        />);
    };

    componentWillUnmount = () => {
        this._progress.destroy();
        console.log("ProgressContainer componentWillUnmount");
    };

    render() {
        return null;
    }
}

const PROGRESS_MAX_WIDTH = 0.8;
const PROGRESS_ANIMATION_DURATION = 200;
const DIMENSION = Dimensions.get('window');
let KEYBOARD_HEIGHT = 0;

// Keyboard.addListener('keyboardDidChangeFrame', function ({ endCoordinates }) {
//     KEYBOARD_HEIGHT = DIMENSION.height - endCoordinates.screenY;
// });

const WINDOW_WIDTH = DIMENSION.width;
const positions = {
    TOP: 20,
    BOTTOM: -20,
    CENTER: 0
};

const durations = {
    LONG: 3500,
    SHORT: 2000
};

let styles = StyleSheet.create({
    defaultStyle: {
        position: 'absolute',
        width: WINDOW_WIDTH,
        justifyContent: 'center',
        alignItems: 'center',
        // backgroundColor: '#00000020',
        backgroundColor: 'transparent',
    },
    containerStyle: {
        paddingHorizontal: 27,
        paddingVertical: 15,
        opacity: 1,
        borderRadius: 4,
        marginHorizontal: WINDOW_WIDTH * ((1 - PROGRESS_MAX_WIDTH) / 2),
        marginBottom: 30,
        justifyContent: 'center',
        alignItems: 'center',
    },
    shadowStyle: {
        shadowColor: '#000',
        shadowOffset: {
            width: 4,
            height: 4
        },
        shadowOpacity: 0.8,
        shadowRadius: 6,
        elevation: 10
    },
    imageStyle: {
        width: 60,
        height: 60
    },
    centerTextStyle: {
        fontSize: 11,
        color: '#FE8D1D',
        textAlign: 'center'
    },
    textStyle: {
        marginTop: 10,
        fontSize: 12,
        color: '#FFF',
        textAlign: 'center'
    },
});

class ProgressContainer extends Component {
    static displayName = 'ProgressContainer';

    static propTypes = {
        ...View.propTypes,
        duration: PropTypes.number,
        visible: PropTypes.bool,
        position: PropTypes.number,
        animation: PropTypes.bool,
        shadow: PropTypes.bool,
        backgroundColor: PropTypes.string,
        shadowColor: PropTypes.string,
        textColor: PropTypes.string,
        delay: PropTypes.number,
        hideOnPress: PropTypes.bool,
        onHide: PropTypes.func,
        onHidden: PropTypes.func,
        onShow: PropTypes.func,
        onShown: PropTypes.func
    };

    static defaultProps = {
        visible: false,
        duration: durations.SHORT,
        animation: true,
        shadow: false,
        position: positions.CENTER,
        delay: 0,
        hideOnPress: true
    };

    constructor() {
        super(...arguments);
        this.state = {
            visible: this.props.visible,
            opacity: new Animated.Value(0),
        };
        this.animatedValue = new Animated.Value(0);
    }

    componentDidMount = () => {
        if (this.state.visible) {
            this._showTimeout = setTimeout(() => this._show(), this.props.delay);
        }
        this.spin()
    };

    componentWillReceiveProps = nextProps => {
        if (nextProps.visible !== this.props.visible) {
            if (nextProps.visible) {
                clearTimeout(this._showTimeout);
                clearTimeout(this._hideTimeout);
                this._showTimeout = setTimeout(() => this._show(), this.props.delay);
            } else {
                this._hide();
            }

            this.setState({
                visible: nextProps.visible
            });
        }
    };

    componentWillUnmount = () => {
        this._hide();
        this.willUnmount = true;
    };

    spin = () => {
        if (this.willUnmount) {
            return;
        }
        this.animatedValue.setValue(0);
        Animated.timing(
            this.animatedValue,
            {
                toValue: 1,
                duration: 1000,
                easing: Easing.linear,
                isInteraction: false,
            }
        ).start(() => this.spin());
    }
    shouldComponentUpdate = (nextProps, nextState) => {
        return this.state.visible !== nextState.visible;
    };

    _animating = false;
    _root = null;
    _hideTimeout = null;
    _showTimeout = null;

    _show = () => {
        clearTimeout(this._showTimeout);
        if (!this._animating) {
            clearTimeout(this._hideTimeout);
            this._animating = true;
            this._root.setNativeProps({
                pointerEvents: 'auto'
            });
            this.props.onShow && this.props.onShow(this.props.siblingManager);
            Animated.timing(this.state.opacity, {
                toValue: 1,
                duration: this.props.animation ? PROGRESS_ANIMATION_DURATION : 0,
                easing: Easing.out(Easing.ease),
                isInteraction: false,
            }).start(({finished}) => {
                if (finished) {
                    this._animating = !finished;
                    if (this.props.onShown) {
                        if (this.props.onShown() && this.props.siblingManager) {
                            this.props.siblingManager.destroy();
                            return;
                        }
                    }
                    if (this.props.duration > 0) {
                        this._hideTimeout = setTimeout(() => this._hide(), this.props.duration);
                    }
                }
            });
        }
    };

    _hide = () => {
        console.log('hide');
        clearTimeout(this._showTimeout);
        clearTimeout(this._hideTimeout);
        if (!this._animating) {
            if (!this._root) {
                return;
            }
            this._root.setNativeProps({
                pointerEvents: 'none'
            });
            this.props.onHide && this.props.onHide(this.props.siblingManager);
            Animated.timing(this.state.opacity, {
                toValue: 0,
                duration: this.props.animation ? PROGRESS_ANIMATION_DURATION : 0,
                easing: Easing.in(Easing.ease),
                isInteraction: false,
            }).start(({finished}) => {
                if (finished) {
                    this._animating = false;
                    this.props.onHidden && this.props.onHidden(this.props.siblingManager);
                }
            });
        }
    };

    render() {

        let interpolatedAnimation = this.animatedValue.interpolate({
            inputRange: [0, 1],
            outputRange: ['0deg', '360deg']
        });

        let {props} = this;
        let offset = props.position;
        let position = offset ? {
            [offset < 0 ? 'bottom' : 'top']: offset < 0 ? (KEYBOARD_HEIGHT - offset) : offset
        } : {
            top: 0,
            bottom: KEYBOARD_HEIGHT
        };

        let text = this.props.children ? (<Text style={[
            styles.textStyle,
            props.textColor && {color: props.textColor}
        ]}>
            {this.props.children}
        </Text>) : null;
        return (this.state.visible || this._animating) ? <View
            style={[
                styles.defaultStyle,
                position
            ]}
            pointerEvents="box-only"
        >

            <TouchableWithoutFeedback
                onPress={this.props.hideOnPress ? this._hide : null}
            >

                <Animated.View
                    style={[
                        styles.containerStyle,
                        props.backgroundColor && {backgroundColor: props.backgroundColor},
                        {
                            opacity: this.state.opacity
                        },
                        props.shadow && styles.shadowStyle,
                        props.shadowColor && {shadowColor: props.shadowColor}
                    ]}
                    pointerEvents="none"
                    ref={ele => this._root = ele}
                >
                    <LoadingView />
                </Animated.View>
            </TouchableWithoutFeedback>
        </View> : null;
    }
}

export default Progress;